# Generated by Django 4.2.11 on 2025-02-13 16:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("h_admin", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="frontend",
            name="account_type",
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



from django.contrib.auth.backends import ModelBackend, BaseBackend
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth import get_user_model
import uuid
from django.conf import settings
import logging
logger = logging.getLogger(__name__)

class SessionEnhancingAuthBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        """
        Override the authenticate method to add custom attributes to the session.
        Authenticate using email rather than username.
        """
        # Use email to look up the user
        if username is not None:
            try:
                user = get_user_model().objects.get(email=username)
            except get_user_model().DoesNotExist:
                return None

            if user.check_password(password):
                if user and user.is_authenticated:
                    extra_fields = ['is_active', 'is_superuser', 'is_staff', 'email']
                    existing_fields = [field for field in extra_fields if hasattr(user, field)]

                    # Set custom session attributes
                    [request.session.__setitem__(f"user_{field}", getattr(user, field)) for field in existing_fields]

                    request.session["account_type"] = settings.AUTH_IDENTIFIER

                return user
        return None



from dj_rest_auth.registration.serializers import RegisterSerializer
from rest_framework import serializers
from django.contrib.auth import get_user_model

User = get_user_model()

class CustomRegisterSerializer(RegisterSerializer):
    username = None
    email = serializers.EmailField(required=True)
    def custom_signup(self, request, user):
        user.email = self.data.get('email')
        user.save()


from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
# from django.conf import settings
# from decouple import config
# account_type = config('ACCOUNTTYPE', default=None)
class CustomTokenObtainPairSerializer(TokenObtainPairSerializer):
    """
    Custom serializer to add extra claims to the JWT and only accept email and password.
    """
    email = serializers.EmailField()
    password = serializers.CharField(write_only=True)

    def __init__(self, *args, **kwargs):
        self.request = kwargs.get('context', {}).get('request', None)
        super().__init__(*args, **kwargs)

        self.fields.pop('username', None)

    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)
        
        # Add custom claims to the token
        token['email'] = user.email
        token['first_name'] = user.first_name
        token['last_name'] = user.last_name
        token['is_superuser'] = user.is_superuser
        return token
    
    # def validate(self, attrs):
    #     """
    #     Validate the email and password and ensure they are correct.
    #     """
    #     email = attrs.get('email')
    #     password = attrs.get('password')

    #     # Validate user using email and password with request context
    #     user = self.authenticate_user( email, password)
    #     if not user:
    #         raise serializers.ValidationError('Invalid email or password.')

    #     # Manually set the username to be the same as email
    #     attrs['username'] = email  # Setting username to email

    #     # Only include necessary user data, NOT the entire CustomUser object
    #     attrs['user'] = {
    #         'email': user.email,
    #         'first_name': user.first_name,
    #         'last_name': user.last_name,
    #         'is_superuser': user.is_superuser,
    #     }

    #     return attrs

    def validate(self, attrs):
        """
        Override validate to authenticate using email instead of username.
        """
        email = attrs.get("email")
        password = attrs.get("password")

        # Retrieve the user model
        User = get_user_model()

        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            raise serializers.ValidationError("Invalid email or password.")

        # Check the password manually
        if not user.check_password(password):
            raise serializers.ValidationError("Invalid email or password.")

        # Use the standard JWT token validation
        data = super().validate({"username": user.get_username(), "password": password})

        return data

    def authenticate_user(self, email, password):
        """
        Custom method to authenticate a user using email and password.
        """
        from django.contrib.auth import authenticate
        
        user = authenticate(self.request, username=email, password=password)
        return user


User = get_user_model()

class BaseEmailCheckSerializer(serializers.Serializer):
    email = serializers.EmailField(required=True)

    def validate_email(self, value):
        return value.lower()

    def validate(self, attrs):
        email = attrs.get('email')
        
        user = User.objects.filter(email=email).first()
        if not user:
            raise serializers.ValidationError({
                "email": f"No account found for this email"
            })
            
        attrs['user'] = user
        return attrs

class EmailCheckSerializer(BaseEmailCheckSerializer):
    pass


from django.contrib import admin
from django.conf import settings
import textwrap

def set_site_id():
    max_length=12
    admin.site.site_title = f"{settings.SERVICE_NAME} site admin"
    admin.site.site_header = textwrap.shorten(f"{settings.SERVICE_NAME} administration", width=max_length, placeholder="...")
    admin.site.index_title = f"{settings.SERVICE_NAME} administration"



from django.shortcuts import render
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from .models import Identity
from .serializers import IdentitySerializer
from django.db.models import Q

class IdentityView(APIView):
    """
    Read-only API for Identity. Returns the single Identity instance.
    Supports dynamic filtering based on query parameters.
    """

    def get(self, request):
        query_filters = Q()

        for key, value in request.query_params.items():
            if key in [field.name for field in Identity._meta.fields]:  # Ensure valid field
                field = Identity._meta.get_field(key)
                if field.is_relation:
                    related_model = field.related_model
                    if hasattr(related_model, "name"):  # Assume the related model has a 'name' field
                        key = f"{key}__name"

                query_filters &= Q(**{key: value})
        
        identity = Identity.objects.filter(query_filters).first()

        if identity:
            serializer = IdentitySerializer(identity)
            return Response(serializer.data)

        return Response({"error": "Identity not found"}, status=status.HTTP_404_NOT_FOUND)



# urls.py
from django.urls import path, include
from rest_framework import routers
from drf_yasg.views import get_schema_view
from drf_yasg import openapi

schema_view = get_schema_view(
   openapi.Info(
      title="My API",
      default_version='v1',
      description="Test description",
      terms_of_service="https://www.google.com/policies/terms/",
      contact=openapi.Contact(email="contact@myapi.local"),
      license=openapi.License(name="MIT License"),
   ),
   public=True,
)


from django.urls import path
from .views import CustomTokenObtainPairView
from rest_framework_simplejwt.views import TokenRefreshView, TokenVerifyView

urlpatterns = [
    # Endpoint to log in and obtain a new token pair (access and refresh tokens)
    path('login/', CustomTokenObtainPairView.as_view(), name='token_obtain_pair'),
    # Endpoint to refresh the access token
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),
    # Endpoint to verify a token's validity
    path('token/verify/', TokenVerifyView.as_view(), name='token_verify'),
]
urlpatterns += [
    path('accounts/', include('allauth.urls')),
     path('swagger/', schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
    path('redoc/', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),

    path('api/auth/', include('dj_rest_auth.urls')),  # Login, Logout, Password Reset
    path('api/auth/registration/', include('dj_rest_auth.registration.urls')),  # Registration
]


from .views import EmailCheckViewSet

urlpatterns += [
    path('api/auth/check-email/', 
         EmailCheckViewSet.as_view({'post': 'create'}), 
         name='check_email'),
]


from django.contrib import admin
from django.conf import settings
from django.utils.html import format_html
from .models import Identity, ServiceType, Frontend

@admin.register(ServiceType)
class ServiceTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'description')  # Display fields in the list view
    search_fields = ('name',)  # Add search functionality for the name field

    def has_add_permission(self, request):
        """Allow adding only if no Identity instance exists."""
        return not ServiceType.objects.exists()


@admin.register(Identity)
class IdentityAdmin(admin.ModelAdmin):
    list_display = ('name', 'service_type', 'enable_admin', 'description', 'service_name_display', 'service_root_display')  # Show in list view
    readonly_fields = ('service_name_display', 'service_root_display')  # Make these fields uneditable in form view

    def has_add_permission(self, request):
        """Allow adding only if no Identity instance exists."""
        return not Identity.objects.exists()

    def service_name_display(self, obj):
        """Display SERVICE_NAME as a read-only field."""
        return format_html("<strong>{}</strong>", settings.SERVICE_NAME)
    service_name_display.short_description = "Service Name"

    def service_root_display(self, obj):
        """Display SERVICE_ROOT as a read-only field."""
        return format_html("<strong>{}</strong>", settings.SERVICE_ROOT)
    service_root_display.short_description = "Service Root"


@admin.register(Frontend)
class FrontendAdmin(admin.ModelAdmin):
    list_display = (
       'account_type', 'flow_title', 'flow_description', 'switch_description', 'sign_in_description',
        'register_description', 'forgot_description', 'reset_description', 'authenticated_route'
    )  # Display all fields in the list view
    search_fields = ('flow_title',)

    def has_add_permission(self, request):
        """Allow adding only if no Frontend instance exists."""
        return not Frontend.objects.exists()


from django.core.management.base import BaseCommand
from houseteli_administrator.h_admin.models import ServiceType, Identity, Frontend
from django.conf import settings

class Command(BaseCommand):
    help = 'Create default ServiceType, Identity, and Frontend instances'

    def handle(self, *args, **kwargs):
        # Create the default 'auth' ServiceType if it doesn't exist
        service_type, created = ServiceType.objects.get_or_create(
            name='auth',
            defaults={'description': 'Authentication Service'}
        )
        
        if created:
            self.stdout.write(self.style.SUCCESS("Created default 'auth' ServiceType"))
        else:
            self.stdout.write(self.style.SUCCESS("'auth' ServiceType already exists"))
        
        # Create the default Frontend instance if it doesn't exist
        if not Frontend.objects.exists():
            frontend = Frontend.objects.create(
                flow_title='Default Flow Title',
                flow_description='Default description for the flow.',
                switch_description='Default switch description.',
                sign_in_description='Default sign-in description.',
                register_description='Default register description.',
                forgot_description='Default forgot description.',
                reset_description='Default reset description.',
                authenticated_route='/dashboard/'
                account_type='Defaut Account Type'
            )
            self.stdout.write(self.style.SUCCESS(f"Created Frontend: {frontend.flow_title}"))
        else:
            self.stdout.write(self.style.SUCCESS("Frontend instance already exists"))

        # Check if the Identity instance exists, if not, create it
        if not Identity.objects.exists():
            identity_name = getattr(settings, 'SERVICE_NAME', 'anon')
            identity_description = f"{identity_name} Service"  # Custom description logic

            identity = Identity.objects.create(
                name=identity_name,
                description=identity_description,
                enable_admin=True,
                service_type=service_type,
                frontend=frontend  # Link the frontend to the identity
            )

            self.stdout.write(self.style.SUCCESS(f"Created Identity: {identity.name}"))
        else:
            self.stdout.write(self.style.SUCCESS("Identity instance already exists"))



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.core.exceptions import ValidationError
import uuid




from django.db import models

class Frontend(models.Model):
    flow_title = models.CharField(max_length=255)
    flow_description = models.TextField()
    switch_description = models.TextField()
    sign_in_description = models.TextField()
    register_description = models.TextField()
    forgot_description = models.TextField()
    reset_description = models.TextField()
    authenticated_route = models.CharField(max_length=255)
    account_type = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return self.flow_title

    class Meta:
        verbose_name_plural = "Frontend"


class ServiceType(models.Model):
    name = models.CharField(max_length=255, unique=True)
    description = models.TextField()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Service Types"

class Identity(models.Model):
    AUTH='auth'
    name = models.CharField(max_length=255)
    description = models.TextField()
    TYPE_CHOICES = [
        (AUTH, 'auth'),
    ]

    # service_type = models.CharField(
    #     max_length=4,
    #     choices=TYPE_CHOICES,
    #     default=AUTH,
    #     editable=False
    # )
    service_type = models.ForeignKey(
        'ServiceType',
        on_delete=models.PROTECT,
        editable=False
    )
    frontend = models.ForeignKey(
        'Frontend',
        on_delete=models.PROTECT,
        editable=False
    )

    enable_admin = models.BooleanField(default=False, verbose_name="Enable Admin")

    def save(self, *args, **kwargs):
        if Identity.objects.exists() and not self.pk:
            raise ValidationError("Only one Identity instance is allowed.")
        super().save(*args, **kwargs)

    def __str__(self):
        return self.name
    
    class Meta:
        verbose_name_plural = "Identity"


from django.shortcuts import render

from rest_framework_simplejwt.views import TokenObtainPairView
from .serializers import CustomTokenObtainPairSerializer

class CustomTokenObtainPairView(TokenObtainPairView):
    """
    View that uses the custom serializer to return a JWT with extra claims.
    """
    serializer_class = CustomTokenObtainPairSerializer


    def get_serializer_context(self):
        """
        Add the request context to the serializer.
        """
        context = super().get_serializer_context()
        context['request'] = self.request  # Pass the request object into the context
        return context




from rest_framework.viewsets import ViewSet
from rest_framework.response import Response
from rest_framework import status
from django.contrib.auth import get_user_model
from allauth.socialaccount.models import SocialAccount
from .serializers import EmailCheckSerializer

User = get_user_model() 

class BaseEmailCheckViewSet(ViewSet):

    def create(self, request):
        serializer = self.serializer_class(data=request.data)

        if serializer.is_valid():
            user = serializer.validated_data['user']
            
            response_data = {
                "name": user.get_full_name() or user.email,
                "email": user.email,
            }

            # Check for social account
            social_account = SocialAccount.objects.filter(user=user).first()
            if social_account:
                response_data["social_providers"] = f"Social account ({social_account.provider})"
            
            return Response(response_data, status=status.HTTP_200_OK)
        
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class EmailCheckViewSet(BaseEmailCheckViewSet):
    serializer_class = EmailCheckSerializer



from rest_framework import serializers
from .models import Identity
from django.conf import settings
from .models import Frontend

class FrontendSerializer(serializers.ModelSerializer):
    class Meta:
        model = Frontend
        fields = '__all__'  # Include all fields of the Frontend model or specify them individually



class IdentitySerializer(serializers.ModelSerializer):
    service_name = serializers.SerializerMethodField()
    service_root = serializers.SerializerMethodField()
    frontend = FrontendSerializer()

    class Meta:
        model = Identity
        fields = ['name', 'service_type', 'enable_admin', 'description', 'service_name', 'service_root', 'frontend']

    def get_service_name(self, obj):
        return settings.SERVICE_NAME

    def get_service_root(self, obj):
        return settings.SERVICE_ROOT





# urls.py
from django.urls import path, include
from rest_framework import routers
from drf_yasg.views import get_schema_view
from drf_yasg import openapi

# You can define your API view, or use the router for DRF views
# For example:
# router = routers.DefaultRouter()
# router.register(r'users', UserViewSet)

schema_view = get_schema_view(
   openapi.Info(
      title="My API",
      default_version='v1',
      description="Test description",
      terms_of_service="https://www.google.com/policies/terms/",
      contact=openapi.Contact(email="contact@myapi.local"),
      license=openapi.License(name="MIT License"),
   ),
   public=True,
)


from django.urls import path
from .views import IdentityView
urlpatterns = [
    path('api/identity/', IdentityView.as_view(), name='identity'),
]


# Generated by Django 4.2.11 on 2025-02-13 13:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Frontend",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("flow_title", models.CharField(max_length=255)),
                ("flow_description", models.TextField()),
                ("switch_description", models.TextField()),
                ("sign_in_description", models.TextField()),
                ("register_description", models.TextField()),
                ("forgot_description", models.TextField()),
                ("reset_description", models.TextField()),
                ("authenticated_route", models.CharField(max_length=255)),
            ],
            options={
                "verbose_name_plural": "Frontend",
            },
        ),
        migrations.CreateModel(
            name="ServiceType",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, unique=True)),
                ("description", models.TextField()),
            ],
            options={
                "verbose_name_plural": "Service Types",
            },
        ),
        migrations.CreateModel(
            name="Identity",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("description", models.TextField()),
                (
                    "enable_admin",
                    models.BooleanField(default=False, verbose_name="Enable Admin"),
                ),
                (
                    "frontend",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="h_admin.frontend",
                    ),
                ),
                (
                    "service_type",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="h_admin.servicetype",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Identity",
            },
        ),
    ]



# second_service/middleware.py
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.conf import settings
from django.shortcuts import redirect
from django.urls import reverse
from django.http import HttpResponseRedirect

class SharedSessionMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.User = get_user_model()
    
    def __call__(self, request):
        if request.session.get("_auth_user_id"):
            # Create a temporary user instance without database lookup
            user = self.User(
                id=request.session["_auth_user_id"],
                is_active=False,
                is_staff=False,
                is_superuser=False
            )
            # extra_fields = ['is_active', 'is_superuser', 'is_staff', 'email', 'user_type']
            extra_fields = ['is_active', 'is_superuser', 'is_staff', 'email']
            # set dditional attributes from session
            for field in extra_fields:
                session_field = f"user_{field}"
                if request.session.get(session_field):
                    setattr(user, field, request.session[session_field])
            request.user = user
            
        else:
            # # login_url = settings.AUTH_SERVICE_ROOT + '/admin'  # Redirect to the authentication service's login endpoint
            # login_url = settings.AUTH_SERVICE_ROOT + f'/admin/login/?next=/{settings.SERVICE_ROOT}admin/'
            # return redirect(login_url) # I'd like to redirect only if accessing admin, other endpoints should return anonymous user
            # request.user = AnonymousUser()

            # Define the admin URL using reverse lookup for the login page
            try:
                admin_login_url = reverse('admin:login')  # Use Django admin login URL
            except Exception as e:
                admin_login_url = settings.AUTH_SERVICE_ROOT + f'/admin/login/?next=/{settings.SERVICE_ROOT}admin/'

            # Check if the request path matches any admin URLs
            # You can use a URL pattern name or manually check the path for certain admin views.
            # Here, I'm assuming that if the request URL matches any admin-related pattern, it should redirect.
            if request.path.startswith(reverse('admin:login')): # or 'admin' in request.path:
                login_url = settings.AUTH_SERVICE_ROOT + f'/admin/login/?next={request.path}'
                return HttpResponseRedirect(login_url)

            # For other endpoints, set an anonymous user
            request.user = AnonymousUser()

        return self.get_response(request)


from allauth.account.adapter import DefaultAccountAdapter
from allauth.utils import build_absolute_uri
from django.conf import settings

class CustomAccountAdapter(DefaultAccountAdapter):

    def get_email_confirmation_url(self, request, emailconfirmation):
        origin = request.META.get('HTTP_ORIGIN', None)
        referer_host = request.META.get('HTTP_REFERER', None)
        # if referer_host:
        #     base_url = referer_host  # Use Referer as the base URL
        if origin:
            base_url = origin
        else:
            base_url = f"{request.scheme}://{request.get_host()}"
        if base_url.endswith('/'):
            base_url = base_url[:-1]
        ret = f"{base_url}/users/verifyemail/?key={emailconfirmation.key}"
        return ret
    
    
    def send_mail(self, template_prefix, email, context):
        """
        Override to customize the password reset URL in the context.
        """
        request = context.get("request")
        if request:
            base_url = settings.FRONTEND_URL
            if base_url.endswith('/'):
                base_url = base_url[:-1]

            # Update the password reset URL in the context
            if "password_reset_url" in context:
                # print(context)
                context["password_reset_url"] = f"{base_url}/users/reset-password/?token={context['token']}&uid={context['uid']}"
        super().send_mail(template_prefix, email, context)